/* Copyright (c) 2010 SpinCore Technologies, Inc.
 *   http://www.spincore.com
 *
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from the
 * use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 * claim that you wrote the original software. If you use this software in a
 * product, an acknowledgment in the product documentation would be appreciated
 * but is not required.
 * 2. Altered source versions must be plainly marked as such, and must not be
 * misrepresented as being the original software.
 * 3. This notice may not be removed or altered from any source distribution.
 */

/**
 * \file Amplitude_Modulation.c
 * \brief This program demonstrates the use of the PBDDS Boards for the generation of the Amplitude Modulated waveforms.
 *\ It takes the Modulation Index, carrier and modulating frequencies as an input from the user.
 * \ingroup mod
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "spinapi.h"

#define CLOCK_FREQ 75.0

// User friendly names for the control bits
#define TX_ENABLE 1
#define TX_DISABLE 0
#define PHASE_RESET 1
#define NO_PHASE_RESET 0
#define DO_TRIGGER 1
#define NO_TRIGGER 0
#define USE_SHAPE 1
#define NO_SHAPE 0

//flexibility of various shapes for the user to specify.

void shape_make_ramp (float *shape_data);
void shape_make_sin (float *shape_data);
void shape_make_sinc (float *shape_data, int lobes);
void shape_make_carrier (float *dds_data);


int detect_boards();
int select_board(int numBoards);
void program_and_start (float fc, float fm);
const double pi = 3.1415926;

int
main (int argc, char **argv)
{
	float fm, fc;
	int numBoards;

	// These arrays will hold the value of the shapes to be used for the
	// shaped pulse feature.
	float dds_data[1024];	// Waveform for the DDS signal itself (used for the carrier)
	float shape_data[1024];	// Waveform for the pulse shape (used for the message signal)

	//Uncommenting the line below will generate a debug log in your current
	//directory that can help debug any problems that you may be experiencing

	//pb_set_debug(1);

	printf("Using SpinAPI library version %s\n", pb_get_version());

	if((numBoards = detect_boards()) > 1) { /*If there is more than one board in the system, have the user specify.*/
		select_board(numBoards); /*Request the board numbet to use from the user*/
	}

	if (pb_init () != 0) {
		printf ("Error initializing board: %s\n", pb_get_error ());
		system("pause");
		return -1;
	}

	pb_set_clock (CLOCK_FREQ);
	pb_set_defaults ();


	printf ("\n");
	printf
    ("This program demonstrates the use of PulseBlasterDDS or RadioProcessor\n"
     "boards in generating an amplitude modulated waveform.\n"
     "Inputs requested are the modulation index, carrier frequency\n"
     "and modulating frequency. The output is the amplitude modulated waveform.\n"
     "By default a sine wave is used for both the modulating and carrier\n"
     "waveform, but any waveform can be used by altering the C code.\n"
     "The TTL outputs are enabled during the pulse and can be used for\n"
     "triggering an oscilloscope. During each period of the TTL pulse,\n"
     "two full periods of the message signal are displayed.\n"
     "\n\n"
     "This is only the basics of what you can do. Please see the \n"
     "manuals for the PulseBlasterDDS and RadioProcessor boards\n"
     "and the source code of this program for more details.\n\n");

	printf ("Press CTRL-C at any time to quit\n\n");

	// Set the contents of the arrays to the desired AM Waveform
	// (see below for the definition of these functions)

	shape_make_sin (shape_data);   //sine waveform is used as the message signal
	                               //changing the function used here can change the shape
                                   //of the message signal
                                   
	shape_make_carrier (dds_data);  // sine wave is used a carrier signal

	// And then load them on the board

	pb_dds_load (shape_data, DEVICE_SHAPE);
	pb_dds_load (dds_data, DEVICE_DDS);

	// Loop continuously, gathering parameters for the demo, and then programming
	// the board appropriately.
	while (1) {
      printf ("Enter frequency of the carrier wave in MHz: ");
      scanf ("%f", &fc);

      printf ("Enter frequency of the modulating wave in MHz: ");
      scanf ("%f", &fm);

      program_and_start (fc,fm);
    }

	// Release control of the board
	pb_close ();

	printf ("\n\n");
	system ("pause");
	return 0;
}

// Programs the PulseBlasterDDS or RadioProcessor board with the user defined
// parameters and outputs the waveform on the analog output and enables
// the TTL pulse. 
void
program_and_start (float fc, float fm)
{
	int start;


	// set the frequency for the sine wave (Carrier Wave)

	pb_start_programming (FREQ_REGS);
	pb_set_freq (fc * MHz);
	pb_stop_programming ();

	// Set the TX phase to 0.0

	pb_start_programming (TX_PHASE_REGS);
	pb_set_phase (0.0);		// in degrees
	pb_stop_programming ();

	printf ("Carrier Frequency: %f\n", fc);
	printf ("Modulating Frequency: %f\n", fm);


    pb_start_programming (PULSE_PROGRAM);

	// outputs the waveform continuously
	// each instruction corresponds to one full period of the message signal
	start =
    pb_inst_radio_shape (0, 0, 0, 0, TX_ENABLE, NO_PHASE_RESET, NO_TRIGGER,
			 USE_SHAPE, 0, 0x0F, CONTINUE, 0, (1/fm) * us);

	pb_inst_radio_shape (0, 0, 0, 0, TX_ENABLE, NO_PHASE_RESET, NO_TRIGGER,
			 USE_SHAPE, 0, 0x00, BRANCH, start, (1/fm) * us);

  pb_stop_programming ();

  pb_start ();
}

int
detect_boards()
{
	int numBoards;

	numBoards = pb_count_boards();	/*Count the number of boards */

    if (numBoards <= 0) {
		printf("No Boards were detected in your system. Verify that the board is firmly secured in the PCI slot.\n\n");
		system("PAUSE");
		exit(-1);
	}

	return numBoards;
}

int
select_board(int numBoards)
{
	int choice;

	do {
		printf("Found %d boards in your system. Which board should be used? (0-%d): ",numBoards, numBoards - 1);
		fflush(stdin);
		scanf("%d", &choice);

		if (choice < 0 || choice >= numBoards) {
			printf("Invalid Board Number (%d).\n", choice);
		}
	} while (choice < 0 || choice >= numBoards);

	pb_select_board(choice);
	printf("Board %d selected.\n", choice);
}

// The following functions show how to build up arrays with different shapes
// for use with the pb_dds_load() function.


// Make the carrier waveform. Here a sine wave is used.
void
shape_make_carrier (float *dds_data)
{
  int i;

  for (i = 0; i < 1024; i++)
    {
      dds_data[i] = sin (2.0 * pi * ((float) i / 1024.0));
    }
}

// Make one period of a sinewave along with the setting of the modulation index from the user
// This is the default message signal used. 
void
shape_make_sin (float *shape_data)
{
  int i;
  
  float MI,A,Ma;
  printf ("Enter the Modulation Index as a value from 0 to 100 percent: ");
  scanf ("%f", &MI);

  Ma = 1.0/((100.0/MI) + 1.0);
  A = Ma*100.0/MI ;

  //Ma and A are set and scaled such that A+Ma*(sin) amplitude does not exceed 1
  //and will give the outputs according to the user specified Modulation Index.


  for (i = 0; i < 1024; i++)
    {
      shape_data[i] = A+(Ma*sin (2.0 * pi * ((float) i / 1024.0)));
    }
}


// Make a sinc shape, an option for the message signal. 
void
shape_make_sinc (float *shape_data, int lobes)
{
  int i;

  float MI,A,Ma;
  printf ("Enter the Modulation Index as a value from 0 to 100 percent: ");
  scanf ("%f", &MI);

  Ma = 1.0/((100.0/MI) + 1.0);
  A = Ma*100.0/MI ;

  double x;
  double scale = (double) lobes * (2.0 * pi);
  for (i = 0; i < 1024; i++)
  {
      x = (double) (i - 512) * scale / 1024.0;
      shape_data[i] = sin (x) / x;
      if ((x) == 0.0)
	  {
	  shape_data[i] = 1.0;
	  }
	  shape_data[i] = A + Ma*shape_data[i];
  }
}


// Make a ramp function. This is an example of a different kind of shape you
// could potentially use for the message signal. 
void
shape_make_ramp (float *shape_data)
{
  int i;
  
  float MI,A,Ma;
  printf ("Enter the Modulation Index as a value from 0 to 100 percent: ");
  scanf ("%f", &MI);

  Ma = 1.0/((100.0/MI) + 1.0);
  A = Ma*100.0/MI ;

  for (i = 0; i < 1024; i++)
    {
      shape_data[i] = (float) i / 1024.0;
      shape_data[i] = A + Ma*shape_data[i];
    }
}






