/* Copyright (c) 2010 SpinCore Technologies, Inc.
 *   http://www.spincore.com
 *
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from the
 * use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 * claim that you wrote the original software. If you use this software in a
 * product, an acknowledgment in the product documentation would be appreciated
 * but is not required.
 * 2. Altered source versions must be plainly marked as such, and must not be
 * misrepresented as being the original software.
 * 3. This notice may not be removed or altered from any source distribution.
 */



/**
 * \file Amplitude_Shift_Keying.c
 * \brief: This program demonstrates an implementation of Ampltude Shift Keying (ASK)
 * \on a PulseBlasterDDS or RadioProcessor board. 
 * \ingroup mod
 */

#include <stdio.h>
#include <math.h>
#include "spinapi.h"

#define CLOCK_FREQ 75.0

// User friendly names for the control bits
#define TX_ENABLE 1
#define TX_DISABLE 0
#define PHASE_RESET 1
#define NO_PHASE_RESET 0
#define DO_TRIGGER 1
#define NO_TRIGGER 0
#define USE_SHAPE 1
#define NO_SHAPE 0


//flexibility of various shapes for the user to specify.
void shape_make_ramp(float *shape_data);
void shape_make_DC(float *shape_data);
void shape_make_sin(float *dds_data);
void shape_make_sinc(float *shape_data, int lobes);


void program_and_start(float amp0, float amp1, float fc);

int main(int argc, char **argv)
{
    float amp1, amp0, fc;

    // These arrays will hold the value of the shapes to be used for the
    // shaped pulse feature.
    float dds_data[1024];   // Waveform for the DDS carrier signal (normally a sinewave)
    float shape_data[1024]; // Waveform for the pulse shape (the message logic sequence)

    printf ("Using spinapi library version %s\n", pb_get_version());
    if(pb_init()) {
        printf ("Error initializing board: %s\n", pb_get_error());
        system("pause");
        return -1;
    }
    pb_set_clock(CLOCK_FREQ);
    pb_set_defaults();


	printf ("\n");
	printf
    ("This program demonstrates an implementation of \n"
     "Amplitude Shift Keying (ASK) on a PulseBlasterDDS or RadioProcessor\n"
     "board. It will output an ASK waveform with a carrier frequency\n"
     "specified by the user. Also, the user will input the corresponding\n"
     "amplitude levels for a logic 1 and a logic 0.\n"
     "The user will then input the length of the desired message and the\n"
     "contents of said message. The carrier waveform is a sine wave.\n"
     "The TTL output show the input message signal with a bit rate\n"
     "of 100 kbps. This is only the basics of what can be done with\n"
     "these boards. Please see their respective manuals and the source\n"
     "code of this program for more details.\n\n");
     

	printf ("Press CTRL-C at any time to quit\n\n");

    // Loop continuously, gathering parameters for the demo, and the programming
    // the board appropriately.
    while (1) {
        printf("Enter the frequency of the carrier in MHz: ");
  	    scanf("%f",&fc);

        printf("\nAmplitudes below should be between 0.0 and 1.0\n");
        printf("Enter the amplitude for a logic 1: ");
  	    scanf("%f",&amp1);

  	    printf("Enter the amplitude for a logic 0: ");
  	    scanf("%f",&amp0);

  	    shape_make_sin(dds_data);
        shape_make_DC(shape_data);
    //shape_make_ramp(dds_data);
    // And then load them on the board
        pb_dds_load(dds_data, DEVICE_SHAPE);
        pb_dds_load(dds_data, DEVICE_DDS);

        program_and_start(amp0, amp1, fc);
    }

	// Release control of the board
	pb_close();

	printf("\n\n");
	system("pause");
	return 0;
}


void program_and_start(float amp0, float amp1, float fc)
{
    int start;
    int ret;
    int length;
    int seq[1024];
    int i;
    float tm=10;

    pb_set_amp(amp0, 0);
    pb_set_amp(amp1, 1);
    // There are two more amplitude registers that can be programmed, but are
    // not used for this demo program.
    //pb_set_amp(0.5, 2);
    //pb_set_amp(0.3, 3);

    // set the frequency for the sine wave (carrier)
	pb_start_programming(FREQ_REGS);
	pb_set_freq(fc*MHz);
	pb_stop_programming();

    // Set the TX phase to 0.0
	pb_start_programming(TX_PHASE_REGS);
	pb_set_phase(0.0); // in degrees
	pb_stop_programming();

    printf("Amplitude for a logic 0: %f\n", amp0);
    printf("Amplitude for a logic 1: %f\n", amp1);
    printf("Freqency of the carrier: %fMHz\n", fc);

    //Taking the length of the logic sequence and accordingly storing the inputs in the array.
    printf("\nPlease enter the length (in bits) of the logic sequence: ");
    scanf("%d",&length);
    for (i=0;i<length;i++)
    {
       printf("Please enter value of bit %d (1 or 0) : ",i+1);
       scanf("%d",&seq[i]);
       if (i == (length - 1))
       {
            printf("\n");
        }
    }

	pb_start_programming (PULSE_PROGRAM);

	// outputs the correct amplitude level based on the value within the logic sequence, each bit is output for one
	// bit period

       for (i=0;i<length;i++)
       {
        if(!seq[i]) //if the ith bit is a zero, output a low pulse
            {
                pb_inst_radio_shape(0,0,0,0, TX_ENABLE, NO_PHASE_RESET, NO_TRIGGER, NO_SHAPE, 0, 0x00,CONTINUE, 0, tm*us);    //output the carrier wave for tm time period with amplitude as assigned for the logic "0"
            }
        else  //if the ith bit is a one, output a high pulse
            {
                pb_inst_radio_shape(0,0,0,0, TX_ENABLE, NO_PHASE_RESET, NO_TRIGGER, NO_SHAPE, 1, 0x0F,CONTINUE, 0, tm*us);    //output the carrier wave for tm time period with amplitude as assigned for the logic "1"

            }
        }



    // Output no pulse for (tm*10) us. reset the phase. TTL outputs off. Execution
    // branches back to the beginning of the pulse program.


    ret =
    pb_inst_radio_shape(0,0,0,0, TX_DISABLE, PHASE_RESET, NO_TRIGGER, NO_SHAPE, 0, 0x00,
                  BRANCH, 0, (tm*10)*us);

    if(ret < 0) {
        printf("Error with third instruction: %s\n", pb_get_error());
        return;
    }

	pb_stop_programming();

	pb_start();
}

// The following functions show how to build up arrays with different shapes
// for use with the pb_dds_load() function.

double pi = 3.1415926;

// Make a sinc shape, for use in generating soft pulses.
void shape_make_sinc(float *shape_data, int lobes)
{
    int i;

    double x;
    double scale = (double)lobes*(2.0*pi);
    for(i=0; i < 1024; i++) {
        x = (double)(i-512)*scale/1024.0;
        shape_data[i] = sin(x)/x;
        if((x) == 0.0) {
            shape_data[i] = 1.0;
        }
    }
}

// Make one period of a sinewave
void shape_make_sin(float *dds_data)
{
    int i;

    for(i=0; i < 1024; i++) {
        dds_data[i] = sin(2.0*pi*((float)i/1024.0));
    }
}

// Make a ramp function. This is an example of a different kind of shape you
// could potentially for a shaped pulse
void shape_make_ramp(float *shape_data)
{
    int i;

    for(i=0; i < 1024; i++) {
        shape_data[i] = (float)i/1024.0;
    }
}


//This functions makes a DC output level, since the amplitude registers are used
//for ASK
void shape_make_DC(float *shape_data)
{
    int i;

    for(i=0; i < 1024; i++) {
        shape_data[i] = 1.0;
    }
}
