/* Copyright (c) 2010 SpinCore Technologies, Inc.
 *   http://www.spincore.com
 *
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from the
 * use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 * claim that you wrote the original software. If you use this software in a
 * product, an acknowledgment in the product documentation would be appreciated
 * but is not required.
 * 2. Altered source versions must be plainly marked as such, and must not be
 * misrepresented as being the original software.
 * 3. This notice may not be removed or altered from any source distribution.
 */

/**
 * \file Frequency_Modulation.c
 * \brief: This program demonstrates the use of the PBDDS-I or 
 * \RadioProcessor Boards for the generation of the Frequency Modulated 
 * \waveforms with carrier and modulating frequencies as specified by the user.
 * \ingroup mod
 */



#include <stdio.h>
#include "spinapi.h"
#include <math.h>

#define CLOCK_FREQ 75.0

// User friendly names for the control bits
#define TX_ENABLE 1
#define TX_DISABLE 0
#define PHASE_RESET 1
#define NO_PHASE_RESET 0
#define DO_TRIGGER 1
#define NO_TRIGGER 0
#define pi 3.14159


void shape_make_sin(float *shape_data);
void shape_make_square(float *shape_data);


void shape_make_data_array(float *data_shape_data,float *rev_data_shape_data);


float N;

int main(int argc, char **argv)
{
	int start;
	int ret;
	int length;
	int i;
	

    pb_set_debug(1);


	  printf ("Using spinapi library version %s\n", pb_get_version());
      if(pb_init()) {
        printf ("Error initializing board: %s\n", pb_get_error());
//        system("pause");
        return -1;
    }
    pb_set_clock(CLOCK_FREQ);
    pb_set_defaults();


	printf ("\n");
	printf
    ("This program demonstrates the use of PBDDS and\n"
     "RadioProcessor boards for the generation of an FM waveform.\n"
     "It will output the FM waveform with the carrier frequency\n"
     "and modulating frequency as specified by the user.\n"
     "Both the carrier and the modulating signal are sine waves.\n"
     "The TTL outputs are enabled during the pulse, and can \n"
     "be used to trigger your oscilloscope. Each time the frequency is\n"
     "changed, there is a new pulse.\n\n"
     "This is only the basics of what you can do.\n"
     "Please see the PulseBlasterDDS and RadioProcessor manuals and the\n"
     "source code for this program for more details.\n\n");

	printf ("Press CTRL-C at any time to quit\n\n");

    printf(
    "\nPlease enter the number of frequency registers you want to use."
    "\nThis is typically 8 or 16 for RadioProcessor boards \n"
    "and 1024 for PBDDS boards: ");
    scanf("%f",&N);
    printf("\nPlease contact SpinCore Technologies if you would like a \ndesign with more frequency registers.\n");
    
    //initialize arrays based on the number of frequency
    //registers available, as specified by the user
    float fc[(int) N],fm,fs; 
    float shape_data [(int) N];
	float data_shape_data [(int) N];
	float rev_data_shape_data [(int) N];
    
    printf("\nPlease enter the carrier frequency in MHz: ");
    scanf("%f",&fs);

    printf("Please enter the modulating frequency in MHz: ");
    scanf("%f",&fm);


	shape_make_sin(shape_data);    //storing the sample values of the sine wave for the N instants
    shape_make_data_array(data_shape_data, rev_data_shape_data);  //dividing the total output range of the amplitude and storing their values for comparison with the input sinusoidal waveform instants.

//According to the comparison between the amplitudes, assign appropriate freqency for that particular instant to be in between the range of (fs+fm/2) to (fs-fm/2) by scaling the total frequency range properly.

    int j;

    for (i=0;i<N;i++)
	{
	    if (shape_data[i]>=0) //For positive amplitude values the frequency should increase for increasing amplitude.
        {
            for (j=N-1;j>=0;j--)
            {
               if (shape_data[i]>data_shape_data[j])
                {
                        fc[i]=((fs) - (fm/2)+ ((2.0*((float)(j/2)/N)*(fm))));
                        break;
                 }
            }
        }

        else //For negative amplitude values the frequency should decrease in the reverse manner i.e for decreasing amplitude the frequency should decrease.
        {
            for (j=N-1;j>=0;j--)
            {
               if (shape_data[i]<rev_data_shape_data[j])
                {
                        fc[i]=((fs) + (fm/2) - ((2.0*((float)((j)/2)/N)*(fm))));
                        break;
                 }
            }
	    }
	}


    //Programming the Frequency registers.

	pb_start_programming(FREQ_REGS);
	for (i=0;i<N;i++)
	{
    	pb_set_freq (fc[i]);
     }
    pb_stop_programming();

	// Write 0.0 degrees to the first phase register of all channels
	pb_start_programming(TX_PHASE_REGS);
	pb_set_phase(0.0);
	pb_stop_programming();

	pb_start_programming(COS_PHASE_REGS);
	pb_set_phase(0.0);
	pb_stop_programming();

	pb_start_programming(SIN_PHASE_REGS);
	pb_set_phase(0.0);
	pb_stop_programming();


	pb_start_programming (PULSE_PROGRAM);

   //Uncomment the below block of code is using 1024 frequency registers and a simple view of the output is desired.
   //This code will output only certain frequencies instead of all 1024 levels. 
   //The below code will not work if 1024 frequency registers are not being used.
   //Again, please contact SpinCore if you would like a board with more frequency registers. 


/*
              pb_inst_radio(1,0,0,0, TX_ENABLE, NO_PHASE_RESET, NO_TRIGGER, 0xFF,CONTINUE, 0, 1000.0*ns);
              pb_inst_radio(1,0,0,0, TX_ENABLE, NO_PHASE_RESET, NO_TRIGGER, 0x00,CONTINUE, 0, 1000.0*ns);
           //   pb_inst_radio(128,0,0,0, TX_ENABLE, NO_PHASE_RESET, NO_TRIGGER, 0xFF,CONTINUE, 0, 1000.0*ns);
           //   pb_inst_radio(128,0,0,0, TX_ENABLE, NO_PHASE_RESET, NO_TRIGGER, 0x00,CONTINUE, 0, 1000.0*ns);
              pb_inst_radio(256,0,0,0, TX_ENABLE, NO_PHASE_RESET, NO_TRIGGER, 0xFF,CONTINUE, 0,1000.0*ns);
              pb_inst_radio(256,0,0,0, TX_ENABLE, NO_PHASE_RESET, NO_TRIGGER, 0x00,CONTINUE, 0,1000.0*ns);
          //    pb_inst_radio(384,0,0,0, TX_ENABLE, NO_PHASE_RESET, NO_TRIGGER, 0xFF,CONTINUE, 0, 1000.0*ns);
          //    pb_inst_radio(384,0,0,0, TX_ENABLE, NO_PHASE_RESET, NO_TRIGGER, 0x00,CONTINUE, 0, 1000.0*ns);
              pb_inst_radio(512,0,0,0, TX_ENABLE, NO_PHASE_RESET, NO_TRIGGER, 0xFF,CONTINUE, 0, 1000.0*ns);
              pb_inst_radio(512,0,0,0, TX_ENABLE, NO_PHASE_RESET, NO_TRIGGER, 0x00,CONTINUE, 0, 1000.0*ns);
          //    pb_inst_radio(640,0,0,0, TX_ENABLE, NO_PHASE_RESET, NO_TRIGGER, 0xFF,CONTINUE, 0, 1000.0*ns);
          //    pb_inst_radio(640,0,0,0, TX_ENABLE, NO_PHASE_RESET, NO_TRIGGER, 0x00,CONTINUE, 0, 1000.0*ns);
              pb_inst_radio(768,0,0,0, TX_ENABLE, NO_PHASE_RESET, NO_TRIGGER, 0xFF,CONTINUE, 0, 1000.0*ns);
              pb_inst_radio(768,0,0,0, TX_ENABLE, NO_PHASE_RESET, NO_TRIGGER, 0x00,CONTINUE, 0, 1000.0*ns);
          //    pb_inst_radio(896,0,0,0, TX_ENABLE, NO_PHASE_RESET, NO_TRIGGER, 0xFF,CONTINUE, 0, 1000.0*ns);
          //    pb_inst_radio(896,0,0,0, TX_ENABLE, NO_PHASE_RESET, NO_TRIGGER, 0x00,CONTINUE, 0, 1000.0*ns);
              pb_inst_radio(1023,0,0,0, TX_ENABLE, NO_PHASE_RESET, NO_TRIGGER, 0xFF,CONTINUE, 0, 1000.0*ns);
              pb_inst_radio(1023,0,0,0, TX_ENABLE, NO_PHASE_RESET, NO_TRIGGER, 0x00,CONTINUE, 0, 1000.0*ns);
*/

//comment the following "for" loop if above code block is uncommented.
    for (i=0;i<(N);i++)
	{
        pb_inst_radio(i,0,0,0, TX_ENABLE, NO_PHASE_RESET, NO_TRIGGER, 0xFF,CONTINUE, 0, (2.0/fs)*us);
        pb_inst_radio(i,0,0,0, TX_ENABLE, NO_PHASE_RESET, NO_TRIGGER, 0x00,CONTINUE, 0, (2.0/fs)*us);

    }

    ret =
            pb_inst_radio(0,0,0,0, TX_DISABLE, PHASE_RESET, NO_TRIGGER, 0x00,BRANCH, 0, (2.0)*ms);

            if(ret < 0) {
                printf("Error with third instruction: %s\n", pb_get_error());
                return -1;
            }


	pb_stop_programming();

	// Trigger program
	pb_start();

	// Release control of the board
	pb_close();
	return 0;
}


//This section of the code samples the message signal of length N as specified
//by the user. For purposes of the above code, two array are used.
//The array rev_data_shape_data is the array data_shape_data in reverse.
void shape_make_data_array(float *data_shape_data, float *rev_data_shape_data)
{

int i=0;
float temp;

temp=(1.0/(N/2));
data_shape_data[0]=-1.0;
rev_data_shape_data[(int)N-1]=-1.0;
    for (i=1;i<N;i++)
    {
        data_shape_data[i]=temp+data_shape_data[i-1];
        rev_data_shape_data[(int)N-i]=data_shape_data[i];
    }

}


void
shape_make_sin (float *shape_data)
{
  int i;

  for (i = 0; i < N; i++)
    {
      shape_data[i] = sin (2.0 * pi * ((float) i / N));
    }
}




