/* Copyright (c) 2010 SpinCore Technologies, Inc.
 *   http://www.spincore.com
 *
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from the
 * use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 * claim that you wrote the original software. If you use this software in a
 * product, an acknowledgment in the product documentation would be appreciated
 * but is not required.
 * 2. Altered source versions must be plainly marked as such, and must not be
 * misrepresented as being the original software.
 * 3. This notice may not be removed or altered from any source distribution.
 */



/*
 * \file Natural_PAM.c
 * \brief This program demonstrates the use of the PulseBlaster DDS-I board for 
 * the generation of the Pulse Amplitude Modulated waveforms
 * with the carrier and modulating signal frequencies as specified by the user/.
 * \ingroup mod
 */

#include <stdio.h>
#include <math.h>
#include "spinapi.h"

#define CLOCK_FREQ 75.0

// User friendly names for the control bits

#define TX_ENABLE 1
#define TX_DISABLE 0
#define PHASE_RESET 1
#define NO_PHASE_RESET 0
#define DO_TRIGGER 1
#define NO_TRIGGER 0
#define USE_SHAPE 1
#define NO_SHAPE 0
#define samples 1024
#define pi 3.14159265


//flexibility of various shapes for the user to specify.

void shape_make_ramp(float *shape_data);

void shape_make_carrier(float *dds_data);

void shape_make_sin(float *shape_data);
void shape_make_sinc(float *shape_data, int lobes);



void program_and_start();


//Global Variables.
float fm=0,fc=0;

int main(int argc, char **argv)
{

    // These arrays will hold the value of the shapes to be used for the
    // shaped pulse feature.
    float dds_data[1024];   // Waveform for the DDS signal itself (normally a square wave for the PAM modulation)
    float shape_data[1024]; // Waveform for the modulating signal(message signal)
    float ratio,temp;

    printf ("Using spinapi library version %s\n", pb_get_version());
    if(pb_init()) {
        printf ("Error initializing board: %s\n", pb_get_error());
        return -1;
    }
    pb_set_clock(CLOCK_FREQ);
    pb_set_defaults();


    printf ("\n\n");
     
    printf
    ("This program demonstrates an implementation of Natural Pulse \n"
     "Amplitude Modulation on PulseBlasterDDS or RadioProcessor boards.\n"
     "The carrier and message signal frequencies are specified by the user.\n"
     "The message signal defaults to a sine wave, and the carrier defaults\n"
     "to a square wave. The TTL outputs show the carrier square wave and\n"
     "can be used to trigger an oscilloscope.\n\n"
     "This is only the basics of what can be done with these boards. Please\n"
     "see their respective manuals and the source code of this program\n"
     "for more details.\n\n");

	printf ("Press CTRL-C at any time to quit\n\n");

    // Loop continuously, gathering parameters for the demo, and then programming
    // the board appropriately.
    while (1) {
        printf("Enter the frequency of the carrier (in MHz): ");
  	    scanf("%f",&fc);

        printf("Enter the frequency of the modulating signal (in MHz): ");
  	    scanf("%f",&fm);
  	    printf("\n");

  	    // Set the contents of the arrays to the desired PAM Waveform
	    // (see below for the definition of these functions)
	    shape_make_sin(shape_data); //use shape_make_ramp here in order to use a ramp message signal instead of a sine wave
	                                //or use shape_make_sinc to make the message signal a sinc pulse
        shape_make_carrier(dds_data);


        // And then load them on the board

        pb_dds_load(shape_data, DEVICE_SHAPE);
        pb_dds_load(dds_data, DEVICE_DDS);

        program_and_start();
    }

    // Release control of the board
	pb_close();

	printf("\n\n");
//	system("pause");
	return 0;
}

// Programs the board with the desired frequencies, as specified by the user
void program_and_start()
{
    int i,start;
    int ret;
    float tm;
    tm=1/fm;

    // set the frequency for the carrier wave (square wave)
	pb_start_programming(FREQ_REGS);
	pb_set_freq(fc*MHz);
	pb_stop_programming();

    // Set the TX phase to 0.0
	pb_start_programming(TX_PHASE_REGS);
	pb_set_phase(0.0); // in degrees
	pb_stop_programming();


//Used only for the cases where Period register is enabled

//setting the set_shape _period register or else it takes the default value as the earlier one set..

//    pb_set_shape_period ((tm*us),0);

	pb_start_programming(PULSE_PROGRAM);


	// set the frequency of the modulating wave by specifying it in a time domain and setting the TTL outputs on for triggering

    start =pb_inst_radio_shape(0,0,0,0, TX_ENABLE, NO_PHASE_RESET, NO_TRIGGER, USE_SHAPE, 0, 0x0F,CONTINUE, 0, tm*us);

        if(start < 0) {
            printf("Error with first instruction: %s\n", pb_get_error());
            return;
        }

     ret =pb_inst_radio_shape(0,0,0,0, TX_ENABLE, NO_PHASE_RESET, NO_TRIGGER, USE_SHAPE, 0, 0x00,BRANCH, start, (tm)*us);

        if(ret < 0) {
            printf("Error with second instruction: %s\n", pb_get_error());
            return;
        }

	pb_stop_programming();

	pb_start();
}




// Make a sinc shape, another wave that can be modulated using PAM

void shape_make_sinc(float *shape_data, int lobes)
{
    int i;

    double x;
    double scale = (double)lobes*(2.0*pi);
    for(i=0; i < 1024; i++) {
        x = (double)(i-512)*scale/1024.0;
        shape_data[i] = sin(x)/x;
        if((x) == 0.0) {
            shape_data[i] = 1.0;
        }
    }
}

// Make one period of a sinewave

void shape_make_sin(float *shape_data)
{
    int i;
    for(i=0; i < 1024; i++)
    {
        shape_data[i] =(sin(2.0*pi*((float)i/1024.0)));
    }

}

// Make a ramp function. This is an example of a different kind of shape you
// could potentially for modulating in PAM

void shape_make_ramp(float *shape_data)
{
    int i;

    for(i=0; i <1024 ; i++)
    {
        shape_data[i] = (-1)*((float)i/1024.0);
    }
}


// Make one period of the carrier wave, which is a square wave.
void shape_make_carrier(float *dds_data)
{
     int i,j;

     for (i=0;i<samples;i++)
        {
            if (i<(samples/2))
            dds_data[i] = 1.0;
            else
            dds_data[i] =0.0;
        }

}
